import 'dart:async';

import 'package:cookie_jar/cookie_jar.dart';
import 'package:dio/dio.dart';
import 'package:dio_cookie_manager/dio_cookie_manager.dart';
import 'package:thyi/src/util/util_log.dart';

class Thyi {
  Dio dio;

  Interceptors get interceptors {
    return dio.interceptors;
  }

  Thyi(String baseUrl, {bool enableLog = true}) {
    isLogEnable = enableLog;
    dio = Dio(BaseOptions(baseUrl: baseUrl));
    interceptors.add(CookieManager(CookieJar()));
  }

}

/// 代表某条api
class ApiMethod {
  String method;
  String path = '';
  Map<String, String> headers = {};
  Map<String, String> fields = {};
  Map<String, String> queries = {};

  ApiMethod(this.method, this.path, {this.headers, this.fields, this.queries});

  Future<String> send(Thyi thyi) {
    if (thyi.interceptors != null && thyi.interceptors.isNotEmpty) {
      thyi.interceptors.forEach((i) => i(this));
    }

    var options = Options();
    var dio = thyi.dio;
    
    if (headers != null && headers.isNotEmpty) {
      options.headers.addAll(headers);
    }

    if (isGet()) {
      logi('url: ${dio.options.baseUrl}, path: ${path}');
      return dio.get<String>(path, queryParameters: queries, options: options)
        .then((response) => _parseResponse(response));
    }
    else if (isPost()) {
      logi('url: ${dio.options.baseUrl}, path: ${path} fileds: ${fields}');
      return dio.post<String>(path, queryParameters: queries, options: options, data: fields)
        .then((response) => _parseResponse(response));
    } else {
      return Future.error('for now unsupport http method; ${method}');
    }

  }  

  String _parseResponse(Response response) {
    var rst = response.data.toString();
    logi("response: $rst");
    return rst;
  }


  bool isGet() {
    return "GET" == method.toUpperCase();
  }

  bool isPost() {
    return "POST" == method.toUpperCase();
  }

}